@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.15.2.1 2001/04/19 08:50:43 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.15.2.2 2001/04/20 14:33:07 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -1143,15 +1143,13 @@ void ip_vs_unbind_dest(struct ip_vs_conn *cp)
                                 atomic_dec(&dest->activeconns);
                         }
                 }
-                
-                /*
-                 *  Decrease the refcnt of the dest, and free the dest
-                 *  if nobody refers to it (refcnt=0).
-                 */
-                if (atomic_dec_and_test(&dest->refcnt)) {
-                        __ip_vs_dst_reset(dest);
-                        kfree(dest);
-                }
+
+		/*
+		 * Simply decrease the refcnt of the dest, because the
+		 * dest will be either in service's destination list
+		 * or in the trash.
+		 */
+                atomic_dec(&dest->refcnt);
         }
 }
 
