@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_core.c,v 1.16 2001/03/22 12:57:46 wensong Exp $
+ * Version:     $Id: ip_vs_core.c,v 1.17 2001/04/04 02:53:11 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -24,10 +24,6 @@
  *
  */
 
-#ifdef MODULE
-#define EXPORT_SYMTAB
-#endif
-
 #include <linux/config.h>
 #include <linux/module.h>
 #include <linux/types.h>
@@ -314,15 +310,8 @@ ip_vs_sched_persist(struct ip_vs_service *svc, struct iphdr *iph)
 			ip_vs_bind_dest(ct, dest);
 			ct->timeout = svc->timeout;
 		} else {
-			/*
-			 * Template found and its destination is available.
-			 */
+                        /* set destination with the found template */
 			dest = ct->dest;
-
-			/*
-			 * Delete its timer so that it can be put back.
-			 */
-			del_sltimer(&ct->timer);
 		}
 		dport = dest->port;
 	} else {
@@ -379,12 +368,8 @@ ip_vs_sched_persist(struct ip_vs_service *svc, struct iphdr *iph)
 			ip_vs_bind_dest(ct, dest);
 			ct->timeout = svc->timeout;
 		} else {
+                        /* set destination with the found template */
 			dest = ct->dest;
-
-			/*
-			 * Delete its timer so that it can be put back.
-			 */
-			del_sltimer(&ct->timer);
 		}
 		dport = portp[1];
 	}
@@ -531,15 +516,15 @@ static int ip_vs_leave(struct ip_vs_service *svc, struct sk_buff *skb)
                 /* bind the bypass_xmit */
                 ip_vs_bind_bypass_xmit(cp);
 
+                /* statistics */
+                ip_vs_in_stats(cp, skb);
+
                 /* set state */
                 ip_vs_set_state(cp, VS_STATE_INPUT, iph, portp); 
 
                 /* transmit the first SYN packet */
                 ret = cp->packet_xmit(skb, cp);
 
-                /* statistics */
-                ip_vs_in_stats(cp, skb);
-                
                 ip_vs_conn_put(cp);
                 return ret;
         }
@@ -679,13 +664,14 @@ static int ip_vs_out_icmp(struct sk_buff **skb_p)
 	/* the TCP/UDP dest port - cannot redo check */
 	pptr[1] = cp->vport;
 
-        ip_vs_out_stats(cp, skb);
-	__ip_vs_conn_put(cp);
-
 	/* And finally the ICMP checksum */
 	icmph->checksum = 0;
 	icmph->checksum = ip_compute_csum((unsigned char *) icmph, len);
 
+        /* do the statistics and put it back */
+        ip_vs_out_stats(cp, skb);
+        ip_vs_conn_put(cp);
+
 	IP_VS_DBG(11, "Forwarding correct outgoing ICMP to "
                   "%u.%u.%u.%u:%d -> %u.%u.%u.%u:%d\n",
 		  NIPQUAD(ciph->saddr), ntohs(pptr[0]),
@@ -784,11 +770,10 @@ static unsigned int ip_vs_out(unsigned int hooknum,
                                   ip_vs_proto_name(iph->protocol), csum);
                         skb->csum = csum_partial(h.raw , doff, csum);
                         csum_ok++;
-                        break;
-
                 case CHECKSUM_HW:
                         if (csum_tcpudp_magic(iph->saddr, iph->daddr, size,
                                               iph->protocol, skb->csum)) {
+				ip_vs_conn_put(cp);
                                 IP_VS_DBG(0, "Outgoing failed %s checksum "
                                           "from %d.%d.%d.%d (size=%d)!\n",
                                           ip_vs_proto_name(iph->protocol),
@@ -913,7 +898,7 @@ unsigned int check_for_ip_vs_out(struct sk_buff **skb_p,
         return NF_ACCEPT;
 }
 
-        
+
 /*
  *	Handle ICMP messages in the outside-to-inside direction (incoming).
  *	Find any that might be relevant, check against existing connections,
@@ -997,7 +982,7 @@ static int ip_vs_in_icmp(struct sk_buff **skb_p)
 	/* The ICMP packet for VS/NAT must be written to correct addresses
            before being forwarded to the right server */
         if ((skb=vs_skb_cow(skb_p, &iph, (unsigned char**)&icmph)) == NULL) {
-		__ip_vs_conn_put(cp);
+                ip_vs_conn_put(cp);
 		return NF_DROP;
 	}
 
@@ -1015,7 +1000,6 @@ static int ip_vs_in_icmp(struct sk_buff **skb_p)
 
 	/* the TCP/UDP source port - cannot redo check */
 	pptr[0] = cp->dport;
-	__ip_vs_conn_put(cp);
 
 	/* And finally the ICMP checksum */
 	icmph->checksum = 0;
@@ -1063,12 +1047,14 @@ static int ip_vs_in_icmp(struct sk_buff **skb_p)
         skb->nf_debug = 1 << NF_IP_LOCAL_OUT;
 #endif /* CONFIG_NETFILTER_DEBUG */
         ip_send(skb);
+        ip_vs_conn_put(cp);
         return NF_STOLEN;
 
   tx_error_icmp:
 	dst_link_failure(skb);
   tx_error:
 	dev_kfree_skb(skb);
+        ip_vs_conn_put(cp);
 	return NF_STOLEN;
 }
 
@@ -1142,10 +1128,9 @@ static unsigned int ip_vs_in(unsigned int hooknum,
 	 */
 	if (cp && cp->dest && !(cp->dest->flags & IP_VS_DEST_F_AVAILABLE)) {
 		/*
-		 * If the dest is not avaiable, don't restart the timer
-		 * of the packet, but silently drop it.
+		 * If the dest is not avaiable, start the timer of the conn
+                 * with its original expires, and silently drop the packet.
 		 */
-		add_sltimer(&cp->timer);
 		__ip_vs_conn_put(cp);
 		return NF_DROP;
 	}
@@ -1162,7 +1147,7 @@ static unsigned int ip_vs_in(unsigned int hooknum,
                         ip_vs_service_put(svc);
 			return NF_DROP;
 		}
-                
+
 		/*
 		 * Let the virtual server select a real server for the
                  * incomming connection, and create a connection entry.
@@ -1178,7 +1163,7 @@ static unsigned int ip_vs_in(unsigned int hooknum,
                 int ret;
 
                 ip_vs_in_stats(cp, skb);
-                ip_vs_set_state(cp, VS_STATE_INPUT, iph, h.portp); 
+                ip_vs_set_state(cp, VS_STATE_INPUT, iph, h.portp);
                 if (cp->packet_xmit)
                         ret = cp->packet_xmit(skb, cp);
                 else {
