@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.15.2.1 2001/04/19 08:50:43 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.15.2.2 2001/04/20 14:33:07 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -539,7 +539,7 @@ ip_vs_lookup_dest(struct ip_vs_service *svc, __u32 daddr, __u16 dport)
  *  scheduling.
  */
 static struct ip_vs_dest *
-ip_vs_get_trash_dest(struct ip_vs_service *svc, __u32 daddr, __u16 dport)
+ip_vs_trash_get_dest(struct ip_vs_service *svc, __u32 daddr, __u16 dport)
 {
 	struct ip_vs_dest *dest;
 	struct list_head *l, *e;
@@ -587,6 +587,31 @@ ip_vs_get_trash_dest(struct ip_vs_service *svc, __u32 daddr, __u16 dport)
 }
 
 
+/*
+ *  Clean up all the destinations in the trash
+ *  Called by the ip_vs_control_cleanup()
+ *
+ *  When the ip_vs_control_clearup is activated by ipvs module exit,
+ *  the service tables must have been flushed and all the connections
+ *  are expired, and the refcnt of each destination in the trash must
+ *  be 1, so we simply release them here.
+ */
+static void ip_vs_trash_cleanup(void)
+{
+	struct ip_vs_dest *dest;
+	struct list_head *l;
+
+	l = &ip_vs_dest_trash;
+
+	while (l->next != l) {
+		dest = list_entry(l->next, struct ip_vs_dest, n_list);
+		list_del(&dest->n_list);
+		__ip_vs_dst_reset(dest);
+		kfree(dest);
+	}
+}
+
+	
 /*
  *  Update a destination in the given service
  */
@@ -718,7 +743,7 @@ static int ip_vs_add_dest(struct ip_vs_service *svc,
 	 * Check if the dest already exists in the trash and
 	 * is from the same service
 	 */
-	dest = ip_vs_get_trash_dest(svc, daddr, dport);
+	dest = ip_vs_trash_get_dest(svc, daddr, dport);
 	if (dest != NULL) {
 		IP_VS_DBG(3, "Get destination %u.%u.%u.%u:%u from trash, "
                           "refcnt=%d, service %u/%u.%u.%u.%u:%u\n",
@@ -1853,6 +1878,7 @@ int __init ip_vs_control_init(void)
 void __exit ip_vs_control_cleanup(void)
 {
         EnterFunction(2);
+	ip_vs_trash_cleanup();
         unregister_sysctl_table(ipv4_vs_table.sysctl_header);
         proc_net_remove("ip_vs_stats");
         proc_net_remove("ip_vs");
